! DART software - Copyright UCAR. This open source software is provided
! by UCAR, "as is", without charge, subject to all terms of use at
! http://www.image.ucar.edu/DAReS/DART/DART_download
!
! $Id$

program sys_sim402

implicit none

! version controlled file description for error handling, do not edit
character(len=256), parameter :: source   = &
   "$URL$"
character(len=32 ), parameter :: revision = "$Revision$"
character(len=128), parameter :: revdate  = "$Date$"

integer n, n_samples, i, j
double precision :: sd_ratio, alpha

write(*, *) 'input group size'
read(*, *) n
write(*, *) 'input sample size'
read(*, *) n_samples

! Loop through a range of values for alpha until alpha is 0.0
do i = 0, 300
   sd_ratio = (i - 1.) / 100.0_r8
   call sub_sim402(sd_ratio, n, n_samples, alpha)
   write(*, *) 'sd_ratio, alpha ', sd_ratio, alpha
!!!   if(alpha <= 0.0) stop
end do

end program sys_sim402



subroutine sub_sim402(sd_ratio, n, n_samples, alpha)

! sys_sim402 adds in an ad hoc measure of additional uncertainty that comes
! from assuming that the sample ratio of s.d. to mean for the regression 
! value is exact by sampling from this distribution with the group size.
! REVISION OF sys_sim102 TO TRY TO DO ABSOLUTE ERROR, NOT SQUARED ERROR
! MINIMIZATION.

! A look at sampling errors for regression coefficients for ensemble filters
! designed to help with localization from multiple ensembles. Notes (if they
! exist) from late August 2003 describe the problem in more detail. Here, we
! ask the question, if a distance to move comes from a distibution with 
! a given ratio of standard deviation to value, when a given size sample
! is used to evaluate this, how should the resulting distance be scaled
! to extract the most useful information.
!

use random_seq_mod, only : random_seq_type, init_random_seq, random_gaussian, &
   twod_gaussians, random_uniform

use sort_mod, only : sort

implicit none

double precision, intent(in) :: sd_ratio
integer, intent(in) :: n, n_samples
double precision, intent(out) :: alpha

type (random_seq_type) :: r
integer, parameter :: num_tries = 1001
double precision :: mean_abs_error(num_tries), base_mean
double precision :: mean_move, mean_move_2, sum_abs_error
double precision, allocatable :: rnum(:), var_hist(:)
double precision :: sample_sigma_y_p
double precision :: c(2, 2), sum_err_var, sigma_y_p, sigma_y_o, sigma_x_p
double precision :: y_truth, y_o, sample_correl, reg_coef, sample_reg_coef
double precision :: sigma_y_u, y_u, sigma_x_u, x_u, sample_x_u, x_error
double precision :: x_error_var, total_err_var, mean(2), correl, est_err_var
double precision :: temp_sum, pe_2_sum, xx, s_xx, s_xx_2, xx_mean, xx_var
double precision, allocatable :: ratio(:), x_var(:)
double precision :: sum_sample_mean, sum_sample_sigma, sum_y_u, sum_sigma_y_u, sum_y_u_2
double precision :: sum_err, sum_err_2, inf_sigma_y_u, var_mean, var_var
double precision :: mean_ratio, sum_ratio, median_ratio
double precision :: garb, table(600)
integer :: i, j, k, index, location(1)

! Initialize repeatable random sequence
call init_random_seq(r) 

! Free parameters are ratio and ensemble size
! Just assume that base value is 1.0 (move is 1.0); can't do case where expected move is 0
! but know the answer to this case (DON"T MOVE!)

! Allocate storage for the sample means
allocate(ratio(n_samples), rnum(n))

! Loop through the number of samples
sum_ratio = 0.0
do i = 1, n_samples

! This first part approximates sampling error from the num groups!!!
! Need an ensemble sized sample of the recursion value
   do j = 1, n
      rnum(j) = random_gaussian(r, dble(1.0), sd_ratio)
   end do


! Compute the sample mean and variance
   call sample_mean_var(rnum, n, base_mean, sample_sigma_y_p)
   ratio(i) =  sqrt(sample_sigma_y_p) / abs(base_mean)
   if(ratio(i) > 10000.0) ratio(i) = 10000.0
   sum_ratio = sum_ratio + ratio(i)

end do

! Now, lookup the value of the mean ratio in the full table
mean_ratio = sum_ratio / n_samples
!write(*, *) 'ratios ', sd_ratio, mean_ratio
!ratio = sort(ratio)
!median_ratio = ratio(n_samples / 2)

! What about the median ratio


! Look up the appropriate value in the full damped table
open(unit = 47, file = 'smoothed_damped_base')
do j = 1, 600
   read(47, *) garb, table(j)
end do

index = int(100 * mean_ratio) + 2
if(index > 600) then
   alpha = 0.0
else
   alpha = table(index)
endif

close(47)

end subroutine sub_sim402

!-----------------------------------------------------
 
subroutine comp_correl(ens, n, correl)
 
implicit none
 
integer, intent(in) :: n
double precision, intent(in) :: ens(2, n)
double precision, intent(out) :: correl
double precision :: sum_x, sum_y, sum_xy, sum_x2, sum_y2


sum_x = sum(ens(2, :))
sum_y = sum(ens(1, :))
sum_xy = sum(ens(2, :) * ens(1, :))
sum_x2 = sum(ens(2, :) * ens(2, :))
 
! Computation of correlation 
sum_y2 = sum(ens(1, :) * ens(1, :))

correl = (n * sum_xy - sum_x * sum_y) / &
   sqrt((n * sum_x2 - sum_x**2) * (n * sum_y2 - sum_y**2))
 
end subroutine comp_correl

!----------------------------------------------------------------

subroutine sample_mean_var(x, n, mean, var)

implicit none

integer, intent(in) :: n
double precision, intent(in) :: x(n)
double precision, intent(out) :: mean, var

double precision :: sx, s_x2

sx = sum(x)
s_x2 = sum(x * x)
mean = sx / n
var = (s_x2 - sx**2 / n) / (n - 1)

end subroutine sample_mean_var

! <next few lines under version control, do not edit>
! $URL$
! $Id$
! $Revision$
! $Date$
